@@ -3,18 +3,35 @@
 # :Created:   mer 02 ago 2017 15:46:11 CEST
 # :Author:    Lele Gaifax <lele@metapensiero.it>
 # :License:   GNU General Public License version 3 or later
-# :Copyright: © 2017, 2018, 2019, 2020, 2021, 2022 Lele Gaifax
+# :Copyright: © 2017, 2018, 2019, 2020, 2021, 2022, 2023 Lele Gaifax
 #
 
 from contextlib import contextmanager
 from io import StringIO
-from re import match
+from re import compile
 from sys import stderr
 
-from . import parse_plpgsql, parse_sql
-from .node import List, Missing, Node, Scalar
+from . import ast, parse_plpgsql, parse_sql
 from .keywords import RESERVED_KEYWORDS, TYPE_FUNC_NAME_KEYWORDS
-from .printers import get_printer_for_node_tag, get_special_function
+from .printers import get_printer_for_node, get_special_function
+
+
+is_simple_name = compile(r'[a-z_][a-z0-9_]*$').match
+"Determine whether a name is simple enough to not require being double quoted."
+
+
+def maybe_double_quote_name(name):
+    """Possibly enclose `name` within double quotes.
+
+    When `name` is not entirely composed by lower case letters, digits or underscores, or it
+    matches one of the PostgreSQL keywords, it must be emitted between double quotes.
+    """
+
+    if ((not is_simple_name(name)
+         or name in RESERVED_KEYWORDS
+         or name in TYPE_FUNC_NAME_KEYWORDS)):
+        name = '"%s"' % name.replace('"', '""')
+    return name
 
 
 class OutputStream(StringIO):
@@ -104,8 +121,6 @@ class OutputStream(StringIO):
 class RawStream(OutputStream):
     """Basic SQL parse tree writer.
 
-    :param int expression_level:
-           start the stream with the given expression level depth, 0 by default
     :param int separate_statements:
            ``1`` by default, tells how many empty lines should separate statements
     :param bool special_functions:
@@ -126,12 +141,11 @@ class RawStream(OutputStream):
     without any adornment.
     """
 
-    def __init__(self, expression_level=0, separate_statements=1, special_functions=False,
+    def __init__(self, separate_statements=1, special_functions=False,
                  comma_at_eoln=False, semicolon_after_last_statement=False,
                  comments=None, remove_pg_catalog_from_functions=False):
         super().__init__()
         self.current_column = 0
-        self.expression_level = expression_level
         self.separate_statements = separate_statements
         self.special_functions = special_functions
         self.comma_at_eoln = comma_at_eoln
@@ -142,7 +156,6 @@ class RawStream(OutputStream):
     def show(self, where=stderr):  # pragma: no cover
         """Emit also current expression_level and a "pointer" showing current_column."""
 
-        where.write('expression_level=%d\n' % self.expression_level)
         super().show(where)
         if self.current_column:
             where.write('-' * (self.current_column-1))
@@ -156,25 +169,24 @@ class RawStream(OutputStream):
         :returns: a string with the equivalent SQL obtained by serializing the syntax tree
 
         The `sql` statement may be either a ``str`` containing the ``SQL`` in textual form, or
-        a :class:`.node.Node` instance, or a :class:`.node.List` instance containing
-        :class:`.node.Node` instances, or a concrete :class:`.ast.Node` instance or a tuple of
-        them.
+        a concrete :class:`.ast.Node` instance or a tuple of them.
         """
 
-        from . import ast
+        from .visitors import Visitor
 
         if isinstance(sql, str):
-            sql = Node(parse_plpgsql(sql) if plpgsql else parse_sql(sql))
-        elif isinstance(sql, Node):
-            sql = (sql,)
+            sql = parse_plpgsql(sql) if plpgsql else parse_sql(sql)
         elif isinstance(sql, ast.Node):
-            sql = (Node(sql),)
-        elif isinstance(sql, tuple) and sql and isinstance(sql[0], ast.Node):
-            sql = (Node(n) for n in sql)
-        elif not isinstance(sql, List):
+            sql = (sql,)
+        elif not (isinstance(sql, tuple) and sql and isinstance(sql[0], ast.Node)):
             raise ValueError("Unexpected value for 'sql', must be either a string,"
-                             " a node.Node instance, a node.List, an ast.Node or tuple of"
-                             " them, got %r" % type(sql))
+                             " an ast.Node or tuple of them, got %r" % type(sql))
+
+        class UpdateAncestors(Visitor):
+            def visit(self, ancestors, node):
+                node.ancestors = ancestors
+
+        UpdateAncestors()(sql)
 
         first = True
         for statement in sql:
@@ -232,18 +244,23 @@ class RawStream(OutputStream):
         yield
 
     @contextmanager
-    def expression(self):
-        "Create a context manager that will wrap subexpressions within parentheses."
+    def expression(self, need_parens):
+        """Create a context manager usable to conditionally wrap something within parentheses.
 
-        self.expression_level += 1
-        if self.expression_level > 1:
+        :param bool need_parens: whether to emit opening and closing parentheses
+
+        This method used to be used for *subexpressions*, but now is used to emit almost
+        anything that goes within parentheses.
+        """
+
+        if need_parens:
             self.write('(')
         yield
-        if self.expression_level > 1:
+        if need_parens:
             self.write(')')
-        self.expression_level -= 1
 
-    def _concat_nodes(self, nodes, sep=' ', are_names=False):
+    def _concat_nodes(self, nodes, sep=' ', are_names=False, item_needs_parens=None):
+
         """Concatenate given `nodes`, using `sep` as the separator.
 
         :param scalars: a sequence of nodes
@@ -251,16 +268,18 @@ class RawStream(OutputStream):
         :param bool are_names:
                whether the nodes are actually *names*, which possibly require to be enclosed
                between double-quotes
+        :param item_needs_parens: either ``None`` or a callable
         :returns: a string
 
         Use a temporary :class:`RawStream` instance to print the list of nodes and return the
         result.
         """
 
-        rawstream = RawStream(expression_level=self.expression_level,
-                              special_functions=self.special_functions,
-                              remove_pg_catalog_from_functions=self.remove_pg_catalog_from_functions)
-        rawstream.print_list(nodes, sep, are_names=are_names, standalone_items=False)
+        rawstream = RawStream(
+            special_functions=self.special_functions,
+            remove_pg_catalog_from_functions=self.remove_pg_catalog_from_functions)
+        rawstream.print_list(nodes, sep, are_names=are_names, standalone_items=False,
+                             item_needs_parens=item_needs_parens)
         return rawstream.getvalue()
 
     def write_quoted_string(self, s):
@@ -268,22 +287,15 @@ class RawStream(OutputStream):
 
         self.write("'%s'" % s.replace("'", "''"))
 
-    def _print_scalar(self, node, is_name, is_symbol):
-        "Print the scalar `node`, special-casing string literals."
+    def _print_scalar(self, value, is_name, is_symbol):
+        "Print the scalar `value`, special-casing string literals."
 
-        value = node.value
-        if is_symbol:
+        is_string = isinstance(value, str)
+        if is_symbol and is_string:
             self.write(value)
-        elif is_name:
-            # The `scalar` represent a name of a column/table/alias: when any of its
-            # characters is not a lower case letter, a digit or underscore, it must be
-            # double quoted
-            if ((not match(r'[a-z_][a-z0-9_]*$', value)
-                 or value in RESERVED_KEYWORDS
-                 or value in TYPE_FUNC_NAME_KEYWORDS)):
-                value = '"%s"' % value.replace('"', '""')
-            self.write(value)
-        elif isinstance(value, str):  # node.parent_node.node_tag == 'String':
+        elif is_name and is_string:
+            self.write(maybe_double_quote_name(value))
+        elif is_string:
             self.write_quoted_string(value)
         else:
             self.write(str(value))
@@ -311,23 +323,29 @@ class RawStream(OutputStream):
     def print_name(self, nodes, sep='.'):
         "Helper method, execute :meth:`print_node` or :meth:`print_list` as needed."
 
-        if isinstance(nodes, (List, list)):
+        if isinstance(nodes, (list, tuple)):
             self.print_list(nodes, sep, standalone_items=False, are_names=True)
-        else:
+        elif isinstance(nodes, ast.Node):
             self.print_node(nodes, is_name=True)
+        else:
+            self._print_scalar(nodes, is_name=True, is_symbol=False)
+        self.separator()
 
     def print_symbol(self, nodes, sep='.'):
         "Helper method, execute :meth:`print_node` or :meth:`print_list` as needed."
 
-        if isinstance(nodes, (List, list)):
+        if isinstance(nodes, (list, tuple)):
             self.print_list(nodes, sep, standalone_items=False, are_names=True, is_symbol=True)
-        else:
+        elif isinstance(nodes, ast.Node):
             self.print_node(nodes, is_name=True, is_symbol=True)
+        else:
+            self._print_scalar(str(nodes), is_name=True, is_symbol=True)
+        self.separator()
 
     def print_node(self, node, is_name=False, is_symbol=False):
         """Lookup the specific printer for the given `node` and execute it.
 
-        :param node: an instance of :class:`~.node.Node` or :class:`~.node.Scalar`
+        :param node: an instance of :class:`~.ast.Node`
         :param bool is_name:
                whether this is a *name* of something, that may need to be double quoted
         :param bool is_symbol:
@@ -340,41 +358,42 @@ class RawStream(OutputStream):
             elif hasattr(node, 'stmt_location'):
                 node_location = getattr(node, 'stmt_location')
             else:
-                node_location = Missing
-            if node_location is not Missing:
+                node_location = None
+            if node_location is not None:
                 nextc = self.comments[0]
-                if nextc.location <= node_location.value:
+                if nextc.location <= node_location:
                     self.print_comment(self.comments.pop(0))
                     while self.comments and self.comments[0].continue_previous:
                         self.print_comment(self.comments.pop(0))
 
-        if isinstance(node, Scalar):
-            self._print_scalar(node, is_name, is_symbol)
-        elif is_name and isinstance(node, (List, list)):
+        if is_name and isinstance(node, (list, tuple)):
             self.print_list(node, '.', standalone_items=False, are_names=True)
-        else:
-            parent_node_tag = node.parent_node and node.parent_node.node_tag
-            printer = get_printer_for_node_tag(parent_node_tag, node.node_tag)
-            if is_name and node.node_tag == 'String':
+        elif isinstance(node, ast.Node):
+            printer = get_printer_for_node(node)
+            if is_name and isinstance(node, ast.String):
                 printer(node, self, is_name=is_name, is_symbol=is_symbol)
             else:
                 printer(node, self)
+        else:
+            self._print_scalar(node, is_name, is_symbol)
+
         self.separator()
 
     def _is_pg_catalog_func(self, items):
         return (
             self.remove_pg_catalog_from_functions
             and len(items) > 1
-            and isinstance(items, List)
-            and items.parent_attribute == 'funcname'
-            and items[0].val.value == 'pg_catalog'
+            and isinstance(items, (list, tuple))
+            and items[0].ancestors.parent.member == 'funcname'
+            and items[0].sval == 'pg_catalog'
             # The list contains all functions that cannot be found without an
             # explicit pg_catalog schema. ie:
             # position(a,b) is invalid but pg_catalog.position(a,b) is fine
-            and items[1].val.value not in ('position', 'xmlexists')
+            and items[1].sval not in ('position', 'xmlexists')
         )
 
-    def _print_items(self, items, sep, newline, are_names=False, is_symbol=False):
+    def _print_items(self, items, sep, newline, are_names=False, is_symbol=False,
+                     item_needs_parens=None):
         first = 1 if self._is_pg_catalog_func(items) else 0
         last = len(items) - 1
         for idx, item in enumerate(items):
@@ -395,13 +414,22 @@ class RawStream(OutputStream):
                         self.write(sep)
                         if sep != '.':
                             self.write(' ')
-            self.print_node(item, is_name=are_names, is_symbol=is_symbol and idx == last)
+            if item is None:
+                self.write('None')
+            else:
+                if item_needs_parens is None:
+                    needs_parens = False
+                else:
+                    needs_parens = item_needs_parens(item)
+                with self.expression(needs_parens):
+                    self.print_node(item, is_name=are_names,
+                                    is_symbol=is_symbol and idx == last)
 
     def print_list(self, nodes, sep=',', relative_indent=None, standalone_items=None,
-                   are_names=False, is_symbol=False):
+                   are_names=False, is_symbol=False, item_needs_parens=None):
         """Execute :meth:`print_node` on all the `nodes`, separating them with `sep`.
 
-        :param nodes: a sequence of :class:`~.node.Node` instances or a single List node
+        :param nodes: a sequence of :class:`~.ast.Node` instances
         :param str sep: the separator between them
         :param bool relative_indent:
                if given, the relative amount of indentation to apply before the first item, by
@@ -413,14 +441,9 @@ class RawStream(OutputStream):
         :param bool is_symbol:
                whether the nodes are actually a *symbol* such as an *operator name*, in which
                case the last one must be printed verbatim (e.g. ``"MySchema".===``)
+        :param item_needs_parens: either ``None`` or a callable
         """
 
-        if isinstance(nodes, Node):  # pragma: no cover
-            if nodes.node_tag != 'List':
-                raise ValueError("Unexpected value for 'nodes', must be either a List instance"
-                                 " or a sequence of Node instances, got %r" % type(nodes))
-            nodes = nodes.items
-
         if relative_indent is None:
             if are_names or is_symbol:
                 relative_indent = 0
@@ -430,21 +453,20 @@ class RawStream(OutputStream):
                                    else 0)
 
         if standalone_items is None:
-            standalone_items = not all(isinstance(n, Node)
-                                       and n.node_tag in ('A_Const', 'ColumnRef',
-                                                          'SetToDefault', 'RangeVar')
+            standalone_items = not all(isinstance(n, (ast.A_Const, ast.ColumnRef,
+                                                      ast.SetToDefault, ast.RangeVar))
                                        for n in nodes)
 
         with self.push_indent(relative_indent):
             self._print_items(nodes, sep, standalone_items, are_names=are_names,
-                              is_symbol=is_symbol)
+                              is_symbol=is_symbol, item_needs_parens=item_needs_parens)
 
     def print_lists(self, lists, sep=',', relative_indent=None, standalone_items=None,
                     are_names=False, sublist_open='(', sublist_close=')', sublist_sep=',',
                     sublist_relative_indent=None):
         """Execute :meth:`print_list` on all the `lists` items.
 
-        :param lists: a sequence of sequences of :class:`~.node.Node` instances
+        :param lists: a sequence of sequences of :class:`~.ast.Node` instances
         :param str sep: passed as is to :meth:`print_list`
         :param bool relative_indent: passed as is to :meth:`print_list`
         :param bool standalone_items: passed as is to :meth:`print_list`
@@ -532,7 +554,7 @@ class IndentedStream(RawStream):
         """Create a context manager that calls :meth:`indent` and :meth:`dedent` around a block
         of code.
 
-        This is just an helper to simplify code that adjust the indentation level:
+        This is just an helper to simplify code that adjusts the indentation level:
 
         .. code-block:: python
 
@@ -585,10 +607,10 @@ class IndentedStream(RawStream):
             self.current_indent = ci
 
     def print_list(self, nodes, sep=',', relative_indent=None, standalone_items=None,
-                   are_names=False, is_symbol=False):
+                   are_names=False, is_symbol=False, item_needs_parens=None):
         """Execute :meth:`print_node` on all the `nodes`, separating them with `sep`.
 
-        :param nodes: a sequence of :class:`~.node.Node` instances
+        :param nodes: a sequence of :class:`~.ast.Node` instances
         :param str sep: the separator between them
         :param bool relative_indent:
                if given, the relative amount of indentation to apply before the first item, by
@@ -600,26 +622,20 @@ class IndentedStream(RawStream):
         :param bool is_symbol:
                whether the nodes are actually an *operator name*, in which case the last one
                must be printed verbatim (such as ``"MySchema".===``)
+        :param item_needs_parens: either ``None`` or a callable
         """
 
-        if isinstance(nodes, Node):  # pragma: no cover
-            if nodes.node_tag != 'List':
-                raise ValueError("Unexpected value for 'nodes', must be either a List instance"
-                                 " or a sequence of Node instances, got %r" % type(nodes))
-            nodes = nodes.items
-
         if standalone_items is None:
             clm = self.compact_lists_margin
             if clm is not None and clm > 0:
-                rawlist = self._concat_nodes(nodes, sep, are_names)
+                rawlist = self._concat_nodes(nodes, sep, are_names, item_needs_parens)
                 if self.current_column + len(rawlist) < clm:
                     self.write(rawlist)
                     return
 
-            standalone_items = not all(
-                (isinstance(n, Node)
-                 and n.node_tag in ('A_Const', 'ColumnRef', 'SetToDefault', 'RangeVar'))
-                for n in nodes)
+            standalone_items = not all(isinstance(n, (ast.A_Const, ast.ColumnRef,
+                                                      ast.SetToDefault, ast.RangeVar))
+                                       for n in nodes)
 
         if (((sep != ',' or not self.comma_at_eoln)
              and len(nodes) > 1
@@ -630,7 +646,8 @@ class IndentedStream(RawStream):
              and standalone_items)):
             self.write(' '*(len(sep) + 1))  # separator added automatically
 
-        super().print_list(nodes, sep, relative_indent, standalone_items, are_names, is_symbol)
+        super().print_list(nodes, sep, relative_indent, standalone_items, are_names, is_symbol,
+                           item_needs_parens)
 
     def write_quoted_string(self, s):
         """Emit the string `s` possibly splitted in successive chunks.
