@@ -3,6 +3,11 @@ select * from sometable
 SELECT *
 FROM sometable
 
+select from sometable
+=
+SELECT
+FROM sometable
+
 select 'foo' as barname,b,c from sometable where c between 1 and 2
 =
 SELECT 'foo' AS barname
@@ -19,6 +24,31 @@ WHERE c BETWEEN 1 AND 2
 :
 {'compact_lists_margin': 80}
 
+select * from fmt a, fmt b where a.id <> b.id and (not a.f or b.f)
+=
+SELECT *
+FROM fmt AS a, fmt AS b
+WHERE a.id <> b.id
+  AND (NOT a.f
+    OR b.f)
+
+select * from fmt a, fmt b where a.id <> b.id and (not a.f or b.f)
+=
+SELECT *
+FROM fmt AS a, fmt AS b
+WHERE a.id <> b.id
+  AND (NOT a.f OR b.f)
+:
+{'compact_lists_margin': 30}
+
+select * from fmt a, fmt b where a.id <> b.id and (not a.f or b.f)
+=
+SELECT *
+FROM fmt AS a, fmt AS b
+WHERE a.id <> b.id AND (NOT a.f OR b.f)
+:
+{'compact_lists_margin': 120}
+
 select 'foo' as barname,b,c,
        (select somevalue
         from othertable
@@ -30,9 +60,9 @@ SELECT 'foo' AS barname
      , c
      , (SELECT somevalue
         FROM othertable
-        WHERE (othertable.x = 1)
-          AND (othertable.y = 2)
-          AND (othertable.z = 3))
+        WHERE othertable.x = 1
+          AND othertable.y = 2
+          AND othertable.z = 3)
 FROM sometable
 WHERE c BETWEEN 1 AND 2
 
@@ -47,7 +77,7 @@ SELECT 'foo' AS barname
      , c
      , (SELECT somevalue
         FROM othertable
-        WHERE (othertable.x = 1) AND (othertable.y = 2) AND (othertable.z = 3))
+        WHERE othertable.x = 1 AND othertable.y = 2 AND othertable.z = 3)
 FROM sometable
 WHERE c BETWEEN 1 AND 2
 :
@@ -64,13 +94,13 @@ SELECT 'foo' AS barname
      , c
      , (SELECT somevalue
         FROM othertable
-        WHERE (othertable.x = 1)
-          AND (othertable.y = 2)
-          AND (othertable.z = 3))
+        WHERE othertable.x = 1
+          AND othertable.y = 2
+          AND othertable.z = 3)
 FROM sometable
 WHERE c BETWEEN 1 AND 2
 :
-{'compact_lists_margin': 75}
+{'compact_lists_margin': 60}
 
 select 'foo' as barname,b,c,
        (select somevalue
@@ -83,9 +113,9 @@ SELECT 'foo' AS barname,
        c,
        (SELECT somevalue
         FROM othertable
-        WHERE (othertable.x = 1)
-          AND (othertable.y = 2)
-          AND (othertable.z = 3))
+        WHERE othertable.x = 1
+          AND othertable.y = 2
+          AND othertable.z = 3)
 FROM sometable
 WHERE c BETWEEN 1 AND 2
 :
@@ -151,7 +181,7 @@ FROM (SELECT c_id
            , count(*) OVER () AS max_rn
       FROM customer
       WHERE c_d_id = 5) AS t
-WHERE rn = (SELECT (floor((random() * max_rn)) + 1))
+WHERE rn = (SELECT floor(random() * max_rn) + 1)
 
 select a.* from a left join (select distinct id from b) as b on a.id = b.id
 =
@@ -167,14 +197,14 @@ from sometable as a
 where not a.bool_flag2 and a.something2 is null or a.other2 = 3
 =
 SELECT a.one
-     , (   (    (NOT a.bool_flag)
-            AND (a.something IS NULL))
-        OR (a.other = 3)) AS foo
-     , a.value1 + ((b.value2 * b.value3)) AS bar
+     ,    (NOT a.bool_flag
+       AND a.something IS NULL)
+       OR a.other = 3 AS foo
+     , a.value1 + (b.value2 * b.value3) AS bar
 FROM sometable AS a
-WHERE ((    (NOT a.bool_flag2)
-        AND (a.something2 IS NULL))
-   OR (a.other2 = 3))
+WHERE (NOT a.bool_flag2
+   AND a.something2 IS NULL)
+   OR a.other2 = 3
 
 select p.name, (select format('[%s] %s', count(*), r.name)
                 from c join r on r.contract_id = c.id
@@ -197,8 +227,8 @@ WHERE p.name LIKE 'lele%'
                    , count(*)
                    , r.name)
        FROM c
-            INNER JOIN r ON (r.contract_id = c.id)
-       WHERE (c.person_id = p.id)) ILIKE 'manager%'
+            INNER JOIN r ON r.contract_id = c.id
+       WHERE c.person_id = p.id) ILIKE 'manager%'
 
 SELECT
 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijkl'
@@ -246,9 +276,9 @@ ORDER BY EXTRACT('year' FROM deliver_date) ASC,
 =
 SELECT *
 FROM manufacturers
-ORDER BY pg_catalog.date_part('year', deliver_date) ASC
-       , pg_catalog.date_part('month', deliver_date) ASC
-       , pg_catalog.date_part('day', deliver_date) ASC
+ORDER BY pg_catalog.extract('year', deliver_date) ASC
+       , pg_catalog.extract('month', deliver_date) ASC
+       , pg_catalog.extract('day', deliver_date) ASC
 
 SELECT * FROM manufacturers
 ORDER BY EXTRACT('year' FROM deliver_date) ASC,
@@ -317,17 +347,17 @@ SELECT id, CASE WHEN (NOT EXISTS (SELECT TRUE FROM aaa WHERE a = 1) or exists (s
 =
 SELECT id
      , CASE
-         WHEN (    ((   (NOT EXISTS (SELECT TRUE
-                                     FROM aaa
-                                     WHERE (a = 1)))
-                     OR EXISTS (SELECT TRUE
-                                FROM bbb
-                                WHERE (b = 2))))
-               AND (id = 1))
+         WHEN (NOT EXISTS (SELECT TRUE
+                           FROM aaa
+                           WHERE a = 1)
+            OR EXISTS (SELECT TRUE
+                       FROM bbb
+                       WHERE b = 2))
+          AND id = 1
            THEN NULL
          ELSE NOT EXISTS (SELECT TRUE
                           FROM ccc
-                          WHERE (c = 1))
+                          WHERE c = 1)
        END
 FROM bar
 
@@ -356,6 +386,18 @@ SELECT CASE a.a
        END
 FROM a
 
+select x from d limit all
+=
+SELECT x
+FROM d
+LIMIT ALL
+
+select x from d limit null
+=
+SELECT x
+FROM d
+LIMIT ALL
+
 (select x from d1 order by y) intersect (select n from d2 group by y limit 3) limit 2
 =
 (SELECT x
@@ -416,3 +458,47 @@ footer
 select 4294967310
 =
 SELECT 4294967310
+
+select t1.*
+from tab1 as t1
+where t1.code like (select t2.code||'.%' from tab1 t2 where t2.id = 'foo')
+=
+SELECT t1.*
+FROM tab1 AS t1
+WHERE t1.code LIKE (SELECT t2.code || '.%'
+                    FROM tab1 AS t2
+                    WHERE t2.id = 'foo')
+
+with recursive t(n) as (values (1) union all select n+1 from t where n < 100)
+select sum(n) from t;
+=
+WITH RECURSIVE t(n)
+  AS (VALUES (1)
+\n\
+      UNION ALL
+\n\
+      SELECT n + 1
+      FROM t
+      WHERE n < 100)
+\n\
+  SELECT sum(n)
+  FROM t
+
+with cte_1 as (select 1), cte_2 as (select * from cte_1)
+select * from cte_2
+=
+WITH cte_1 AS (SELECT 1)
+\n\
+   , cte_2 AS (SELECT *
+               FROM cte_1)
+\n\
+  SELECT *
+  FROM cte_2
+
+SELECT 1 FROM ONLY "public"."produit" x WHERE "produit_id"
+OPERATOR(pg_catalog.=) $1 FOR KEY SHARE OF x
+=
+SELECT 1
+FROM ONLY public.produit AS x
+WHERE produit_id OPERATOR(pg_catalog.=) $1
+FOR KEY SHARE OF x
